"""
Operating machine learning pipelines (basic)
============================================

You can create pipelines using the following operators:

    * Chain -->  It creates a sequential chain of components

    * Shuffle --> It shuffles the componensts order

    * Select --> It selects one of the given componets

"""

################################################################################
# Importing the required packages
#

import numpy as np

from pjml.operator.chain import Chain
from pjml.operator.select import Select
from pjml.operator.shuffle import Shuffle
from pjpy.modeling.supervised.classifier.dt import DT
from pjpy.modeling.supervised.classifier.svmc import SVMC
from pjpy.processing.feature.reductor.pca import PCA
from pjpy.processing.feature.scaler.minmax import MinMax

np.random.seed(0)

################################################################################
# Using `Chain`
# -------------
# The ``Chain`` is an operator that concatenates other components in a sequence.
#

exp = Chain(SVMC(), DT())
print(exp)

# You can also use the python operator ``*``

exp = SVMC() * DT()
print(exp)

################################################################################
# Using `Shuffle`
# ---------------
# The ``Select`` is an operator that works like a bifurcation, where only one of
# the components will be selected.
#

exp = Shuffle(PCA(), MinMax())
print(exp)

################################################################################
# You can also use the python operator ``@``
#

exp = PCA() @ MinMax()
print(exp)

################################################################################
# Using `Select`
# --------------
#
# The ``Shuffle`` is an operator that concatenate components in a sequence, but
# the order is not maintained.
#

exp = Select(SVMC(), DT())
print(exp)

################################################################################
# You can also use the python operator ``+``
#

exp = SVMC() + DT()
print(exp)

################################################################################
# Using them all:
# ---------------
#
# Using these simple operations, you can create diverse kind of pipelines
# to represent an end-to-end machine learning pipeline.
#

exp = Chain(Shuffle(PCA(), MinMax()), Select(SVMC() + DT()))
print(exp)

################################################################################
# You can also use python operators
#

exp = PCA() @ MinMax() * (SVMC() + DT())
print(exp)
