"""
Creating an end-to-end pipelines
================================
Let's create an end-to-end machine learning pipeline.

"""

################################################################################
# Importing the required packages
#

import numpy as np

from pjml.data.communication.report import Report
from pjml.data.evaluation.metric import Metric
from pjml.data.flow.file import File
from pjml.operator.pipeline import Pipeline
from pjml.stream.expand.partition import Partition
from pjml.stream.reduce.reduce import Reduce
from pjml.stream.reduce.summ import Summ
from pjml.stream.transform.map import Map
from pjpy.modeling.supervised.classifier.svmc import SVMC
from pjpy.processing.feature.reductor.pca import PCA
from pjpy.processing.feature.scaler.minmax import MinMax

np.random.seed(0)

################################################################################
# First, we create a machine learning expression.
#

exp = Pipeline(MinMax(), PCA(), SVMC())

################################################################################
# Let's look at the sequence of operations and the hyperparameter values.
#

print(exp)

################################################################################
# Defined our machine learning expression, we will create an end-to-end
# pipeline.
#

pipeline = Pipeline(
    File("../data/iris.arff"),
    Partition(),
    Map(exp, Metric()),
    Summ(),
    Reduce(),
    Report(),
)

################################################################################
# or using only python operators
#

pipeline = (
    File("../data/iris.arff")
    * Partition()
    * Map(exp * Metric())
    * Summ(function="mean")
    * Reduce()
    * Report("Mean S: $S")
)

################################################################################
# This pipeline represents and end-to-end machine learning experiment.
#

print(pipeline)
